"""
Module for Jenkinsapi Invocation object
"""

import time
import datetime
from jenkinsapi.custom_exceptions import UnknownQueueItem, TimeOut


class Invocation(object):
    """
    Represents the state and consequences of a single attempt to start a job.
    This class provides a context manager which is intended to watch the state of the job
    before and after the invoke. It will detect whether a process got queued, launched
    or whether nothing at all happened.

    An instance of this object will be returned by job.invoke()
    """

    def __init__(self, job):
        self.job = job
        self.initial_builds = None
        self.queue_item = None
        self.build_number = None

    def __enter__(self):
        """
        Start watching the job
        """
        self.job.poll()
        self.initial_builds = set(self.job.get_build_dict().keys())

    def __exit__(self, type_, value, traceback):
        """
        Finish watching the job - it will track which new queue items or builds have
        been created as a consequence of invoking the job.
        """
        self.job.poll()
        newly_created_builds = set(self.job.get_build_dict().keys())
        if newly_created_builds:
            self.build_number = newly_created_builds.pop()
        else:
            try:
                self.queue_item = self.job.get_queue_item()
                self.build_number = self.job.get_next_build_number()
            except UnknownQueueItem:
                pass

    def get_build_number(self):
        """
        If this job is building or complete then provide it's build-number
        """
        self.job.poll()
        self.build_number = self.job.get_last_buildnumber()
        return self.build_number

    def get_build(self):
        return self.job[self.get_build_number()]

    def block_until_not_queued(self, timeout, delay):
        # self.__block(lambda: self.is_queued(), False, timeout, delay)
        self.__block(self.is_queued, False, timeout, delay)

    def block_until_completed(self, timeout, delay):
        # self.__block(lambda: self.is_running(), False, timeout, delay)
        self.__block(self.is_running, False, timeout, delay)

    @staticmethod
    def __block(fn, expectation, timeout, delay=2):
        startTime = datetime.datetime.now()
        endTime = startTime + datetime.timedelta(seconds=timeout)
        while True:
            if fn() == expectation:
                break
            else:
                time.sleep(delay)
            if datetime.datetime.now() > endTime:
                raise TimeOut()

    def block(self, until='completed', timeout=200, delay=2):
        """
        Block this item until a condition is met.
        Setting until to 'running' blocks the item until it is running (i.e. it's no longer queued)
        """
        assert until in ['completed', 'not_queued'], 'Unknown block condition: %s' % until
        self.block_until_not_queued(timeout, delay)
        if until == 'completed':
            self.block_until_completed(timeout, delay)

    def stop(self):
        """
        Stop this item, whether it is on the queue or blocked.
        """
        self.get_build().stop()

    def is_queued(self):
        """
        Returns True if this item is on the queue
        """
        return self.job.is_queued()

    def is_running(self):
        """
        Returns True if this item is executing now
        Returns False if this item has completed
        or has not yet executed.
        """
        try:
            return self.get_build().is_running()
        except KeyError:
            # This item has not yet executed
            return False

    def is_queued_or_running(self):
        return self.is_queued() or self.is_running()

    def get_queue_item(self):
        """
        If the item is queued it will return that QueueItem, otherwise it will
        raise an exception.
        """
        return self.queue_item



import SimpleHTTPServer
import SocketServer
import logging
import cgi

PORT = 8080


class ServerHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

    def do_GET(self):
        logging.error(self.headers)
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

    def do_POST(self):
        logging.error(self.headers)
        form = cgi.FieldStorage(
            fp=self.rfile,
            headers=self.headers,
            environ={'REQUEST_METHOD': 'POST',
                     'CONTENT_TYPE': self.headers['Content-Type'],
                     })
        for item in form.list:
            logging.error(item)
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

Handler = ServerHandler

httpd = SocketServer.TCPServer(("", PORT), Handler)

print "serving at port", PORT
httpd.serve_forever()



"""
Module for custom_exceptions specialized for jenkinsapi
"""


class JenkinsAPIException(Exception):
    """
    Base class for all errors
    """
    pass


class NotFound(JenkinsAPIException):
    """
    Resource cannot be found
    """
    pass


class ArtifactsMissing(NotFound):
    """
    Cannot find a build with all of the required artifacts.
    """
    pass


class UnknownJob(KeyError, NotFound):
    """
    Jenkins does not recognize the job requested.
    """
    pass


class UnknownView(KeyError, NotFound):
    """
    Jenkins does not recognize the view requested.
    """
    pass


class UnknownNode(KeyError, NotFound):
    """
    Jenkins does not recognize the node requested.
    """
    pass


class UnknownQueueItem(KeyError, NotFound):
    """
    Jenkins does not recognize the requested queue item
    """
    pass


class NoBuildData(NotFound):
    """
    A job has no build data.
    """
    pass


class ArtifactBroken(JenkinsAPIException):
    """
    An artifact is broken, wrong
    """
    pass


class TimeOut(JenkinsAPIException):
    """
    Some jobs have taken too long to complete.
    """
    pass


class WillNotBuild(JenkinsAPIException):
    """
    Cannot trigger a new build.
    """
    pass


class NoResults(JenkinsAPIException):
    """
    A build did not publish any results.
    """
    pass


class FailedNoResults(NoResults):
    """
    A build did not publish any results because it failed
    """
    pass


class BadURL(ValueError, JenkinsAPIException):
    """
    A URL appears to be broken
    """
    pass


class NotAuthorized(JenkinsAPIException):
    """Not Authorized to access resource"""
    # Usually thrown when we get a 403 returned
    pass


class NotSupportSCM(JenkinsAPIException):
    """
    It's a SCM that does not supported by current version of jenkinsapi
    """
    pass


class NotConfiguredSCM(JenkinsAPIException):
    """
    It's a job that doesn't have configured SCM
    """
    pass


class NotInQueue(JenkinsAPIException):
    """
    It's a job that is not in the queue
    """
    pass



'''
System tests for `jenkinsapi.jenkins` module.
'''
import unittest
import time
from jenkinsapi.build import Build
from jenkinsapi.invocation import Invocation
from jenkinsapi_tests.systests.base import BaseSystemTest
from jenkinsapi_tests.test_utils.random_strings import random_string
from jenkinsapi_tests.systests.job_configs import LONG_RUNNING_JOB
from jenkinsapi_tests.systests.job_configs import SHORTISH_JOB, EMPTY_JOB


class TestInvocation(BaseSystemTest):

    def test_invocation_object(self):
        job_name = 'create_%s' % random_string()
        job = self.jenkins.create_job(job_name, LONG_RUNNING_JOB)
        ii = job.invoke(invoke_pre_check_delay=7)
        self.assertIsInstance(ii, Invocation)
        # Let Jenkins catchup
        time.sleep(3)
        self.assertTrue(ii.is_queued_or_running())
        self.assertEquals(ii.get_build_number(), 1)

    def test_get_block_until_build_running(self):
        job_name = 'create_%s' % random_string()
        job = self.jenkins.create_job(job_name, LONG_RUNNING_JOB)
        ii = job.invoke(invoke_pre_check_delay=7)
        time.sleep(3)
        bn = ii.get_build_number()
        self.assertIsInstance(bn, int)
        ii.block(until='not_queued')
        self.assertTrue(ii.is_running())
        b = ii.get_build()
        self.assertIsInstance(b, Build)
        ii.stop()
        self.assertFalse(ii.is_running())
        self.assertIsInstance(ii.get_build().get_console(), str)
        self.assertIn('Started by user', ii.get_build().get_console())

    def test_get_block_until_build_complete(self):
        job_name = 'create_%s' % random_string()
        job = self.jenkins.create_job(job_name, SHORTISH_JOB)
        ii = job.invoke()
        ii.block(until='completed')
        self.assertFalse(ii.is_running())

    def test_multiple_invocations_and_get_last_build(self):
        job_name = 'create_%s' % random_string()

        job = self.jenkins.create_job(job_name, SHORTISH_JOB)

        for _ in range(3):
            ii = job.invoke()
            ii.block(until='completed')

        build_number = job.get_last_good_buildnumber()
        self.assertEquals(build_number, 3)

        build = job.get_build(build_number)
        self.assertIsInstance(build, Build)

    def test_multiple_invocations_and_get_build_number(self):
        job_name = 'create_%s' % random_string()

        job = self.jenkins.create_job(job_name, EMPTY_JOB)

        for invocation in range(3):
            ii = job.invoke()
            ii.block(until='completed')
            build_number = ii.get_build_number()
            self.assertEquals(build_number, invocation + 1)


if __name__ == '__main__':
    unittest.main()



"""
Module for JenkinsBase class
"""

import ast
import logging
from jenkinsapi import config
from jenkinsapi.custom_exceptions import JenkinsAPIException
log = logging.getLogger(__name__)


class JenkinsBase(object):
    """
    This appears to be the base object that all other jenkins objects are inherited from
    """
    RETRY_ATTEMPTS = 1

    def __repr__(self):
        return """<%s.%s %s>""" % (self.__class__.__module__,
                                   self.__class__.__name__,
                                   str(self))

    def __str__(self):
        raise NotImplementedError

    def __init__(self, baseurl, poll=True):
        """
        Initialize a jenkins connection
        """
        self._data = None
        self.baseurl = self.strip_trailing_slash(baseurl)
        if poll:
            self.poll()

    def get_jenkins_obj(self):
        raise NotImplementedError('Please implement this method on %s' % self.__class__.__name__)

    def __eq__(self, other):
        """
        Return true if the other object represents a connection to the same server
        """
        if not isinstance(other, self.__class__):
            return False
        if not other.baseurl == self.baseurl:
            return False
        return True

    @classmethod
    def strip_trailing_slash(cls, url):
        while url.endswith('/'):
            url = url[:-1]
        return url

    def poll(self):
        self._data = self._poll()

    def _poll(self):
        url = self.python_api_url(self.baseurl)
        return self.get_data(url)

    def get_data(self, url, params=None):
        requester = self.get_jenkins_obj().requester
        response = requester.get_url(url, params)
        try:
            return ast.literal_eval(response.text)
        except Exception:
            log.exception('Inappropriate content found at %s', url)
            raise JenkinsAPIException('Cannot parse %s' % response.content)

    @classmethod
    def python_api_url(cls, url):
        if url.endswith(config.JENKINS_API):
            return url
        else:
            if url.endswith(r"/"):
                fmt = "%s%s"
            else:
                fmt = "%s/%s"
            return fmt % (url, config.JENKINS_API)



import logging
logging.basicConfig()

from jenkinsapi.jenkins import Jenkins
from pkg_resources import resource_string
J = Jenkins('http://localhost:8080')
jobName = 'foo_job2'
xml = resource_string('examples', 'addjob.xml')

print xml

j = J.create_job(jobname=jobName, config=xml)

j2 = J[jobName]
print j

# Delete job
J.delete_job(jobName)



"""
A lower level example of how we login with authentication
"""

from jenkinsapi import jenkins


J = jenkins.Jenkins("http://localhost:8080", username="sal", password="foobar")
J.poll()

print J.items()



"""
This module implements the Jobs class, which is intended to be a container-like
interface for all of the jobs defined on a single Jenkins server.
"""
import logging
from jenkinsapi.job import Job
from jenkinsapi.custom_exceptions import JenkinsAPIException, UnknownJob

log = logging.getLogger(__name__)


class Jobs(object):
    """
    This class provides a container-like API which gives
    access to all jobs defined on the Jenkins server. It behaves
    like a dict in which keys are Job-names and values are actual
    jenkinsapi.Job objects.
    """
    def __init__(self, jenkins):
        self.jenkins = jenkins

    def __len__(self):
        return len(self.keys)

    def __delitem__(self, job_name):
        """
        Delete a job by name
        :param job_name: name of a exist job, str
        """
        if job_name in self:
            delete_job_url = self[job_name].get_delete_url()
            self.jenkins.requester.post_and_confirm_status(
                delete_job_url,
                data='some random bytes...'
            )
            self.jenkins.poll()

    def __setitem__(self, key, value):
        raise NotImplementedError()

    def __getitem__(self, job_name):
        for row in self.jenkins._data.get('jobs', []):
            if row['name'] == job_name:
                return Job(
                    row['url'],
                    row['name'],
                    self.jenkins)

        raise UnknownJob(job_name)

    def iteritems(self):
        """
        Get the names & objects for all jobs
        """
        self.jenkins.poll()
        for row in self.jenkins._data.get('jobs', []):
            name = row['name']
            url = row['url']

            yield name, Job(url, name, self.jenkins)

    def __contains__(self, job_name):
        """
        True if job_name is the name of a defined job
        """
        return job_name in self.keys()

    def iterkeys(self):
        """
        Get the names of all available views
        """
        for row in self.jenkins._data.get('jobs', []):
            yield row['name']

    def keys(self):
        """
        Return a list of the names of all jobs
        """
        return list(self.iterkeys())

    def create(self, job_name, config):
        """
        Create a job
        :param jobname: name of new job, str
        :param config: configuration of new job, xml
        :return: new Job obj
        """
        if job_name in self:
            return self[job_name]

        params = {'name': job_name}
        if isinstance(config, unicode):
            config = str(config)
        self.jenkins.requester.post_xml_and_confirm_status(
            self.jenkins.get_create_url(),
            data=config,
            params=params
        )
        self.jenkins.poll()
        if job_name not in self:
            raise JenkinsAPIException('Cannot create job %s' % job_name)

        return self[job_name]

    def copy(self, job_name, new_job_name):
        """
        Copy a job
        :param job_name: name of a exist job, str
        :param new_job_name: name of new job, str
        :return: new Job obj
        """
        params = {'name': new_job_name,
                  'mode': 'copy',
                  'from': job_name}

        self.jenkins.requester.post_and_confirm_status(
            self.jenkins.get_create_url(),
            params=params,
            data='')
        self.jenkins.poll()
        return self[new_job_name]

    def rename(self, job_name, new_job_name):
        """
        Rename a job
        :param job_name: name of a exist job, str
        :param new_job_name: name of new job, str
        :return: new Job obj
        """
        params = {'newName': new_job_name}
        rename_job_url = self[job_name].get_rename_url()
        self.jenkins.requester.post_and_confirm_status(
            rename_job_url, params=params, data='')
        self.jenkins.poll()
        return self[new_job_name]

    def build(self, job_name, params):
        assert isinstance(params, dict)
        self[job_name].invoke(build_params=params)



"""
jenkinsapi plugins
"""

import logging

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.plugin import Plugin


log = logging.getLogger(__name__)


class Plugins(JenkinsBase):
    """
    Plugins class for jenkinsapi
    """
    def __init__(self, url, jenkins_obj):
        self.jenkins_obj = jenkins_obj
        JenkinsBase.__init__(self, url)
        # print 'DEBUG: Plugins._data=', self._data

    def get_jenkins_obj(self):
        return self.jenkins_obj

    def _poll(self):
        return self.get_data(self.baseurl)

    def keys(self):
        return self.get_plugins_dict().keys()

    def iteritems(self):
        return self._get_plugins()

    def values(self):
        return [a[1] for a in self.iteritems()]

    def _get_plugins(self):
        if not 'plugins' in self._data:
            pass
        else:
            for p_dict in self._data["plugins"]:
                yield p_dict["shortName"], Plugin(p_dict)

    def get_plugins_dict(self):
        return dict(self._get_plugins())

    def __len__(self):
        return len(self.get_plugins_dict().keys())

    def __getitem__(self, plugin_name):
        return self.get_plugins_dict().get(plugin_name, None)

    def __contains__(self, plugin_name):
        """
        True if plugin_name is the name of a defined plugin
        """
        return plugin_name in self.keys()

    def __str__(self):
        plugins = [plugin["shortName"] for plugin in self._data.get("plugins", [])]
        return str(sorted(plugins))



"""
Artifacts can be used to represent data created as a side-effect of running a Jenkins build.

Artifacts are files which are associated with a single build. A build can have any number of
artifacts associated with it.

This module provides a class called Artifact which allows you to download objects from the server
and also access them as a stream.
"""
import os
import logging
import hashlib

from jenkinsapi.fingerprint import Fingerprint
from jenkinsapi.custom_exceptions import ArtifactBroken

log = logging.getLogger(__name__)


class Artifact(object):
    """
    Represents a single Jenkins artifact, usually some kind of file
    generated as a by-product of executing a Jenkins build.
    """

    def __init__(self, filename, url, build):
        self.filename = filename
        self.url = url
        self.build = build

    def save(self, fspath):
        """
        Save the artifact to an explicit path. The containing directory must exist.
        Returns a reference to the file which has just been writen to.

        :param fspath: full pathname including the filename, str
        :return: filepath
        """
        log.info(msg="Saving artifact @ %s to %s" % (self.url, fspath))
        if not fspath.endswith(self.filename):
            log.warn(msg="Attempt to change the filename of artifact %s on save." % self.filename)
        if os.path.exists(fspath):
            if self.build:
                try:
                    if self._verify_download(fspath):
                        log.info(msg="Local copy of %s is already up to date." % self.filename)
                        return fspath
                except ArtifactBroken:
                    log.info("Jenkins artifact could not be identified.")
            else:
                log.info("This file did not originate from Jenkins, so cannot check.")
        else:
            log.info("Local file is missing, downloading new.")
        filepath = self._do_download(fspath)
        try:
            self._verify_download(filepath)
        except ArtifactBroken:
            log.warning("fingerprint of the downloaded artifact could not be verified")
        return fspath

    def get_jenkins_obj(self):
        return self.build.get_jenkins_obj()

    def get_data(self):
        """
        Grab the text of the artifact
        """
        response = self.get_jenkins_obj().requester.get_and_confirm_status(self.url)
        return response.content

    def _do_download(self, fspath):
        """
        Download the the artifact to a path.
        """
        with open(fspath, "wb") as out:
            out.write(self.get_data())
        return fspath

    def _verify_download(self, fspath):
        """
        Verify that a downloaded object has a valid fingerprint.
        """
        local_md5 = self._md5sum(fspath)
        fp = Fingerprint(self.build.job.jenkins.baseurl, local_md5, self.build.job.jenkins)
        return fp.validate_for_build(os.path.basename(fspath), self.build.job.name, self.build.buildno)

    def _md5sum(self, fspath, chunksize=2 ** 20):
        """
        A MD5 hashing function intended to produce the same results as that used by
        Jenkins.
        """
        md5 = hashlib.md5()
        try:
            with open(fspath, 'rb') as f:
                for chunk in iter(lambda: f.read(chunksize), ''):
                    md5.update(chunk)
        except:
            raise
        return md5.hexdigest()

    def save_to_dir(self, dirpath):
        """
        Save the artifact to a folder. The containing directory must be exist, but use the artifact's
        default filename.
        """
        assert os.path.exists(dirpath)
        assert os.path.isdir(dirpath)
        outputfilepath = os.path.join(dirpath, self.filename)
        return self.save(outputfilepath)

    def __repr__(self):
        """
        Produce a handy repr-string.
        """
        return """<%s.%s %s>""" % (self.__class__.__module__, self.__class__.__name__, self.url)



from jenkinsapi.view import View
from jenkinsapi.jenkins import Jenkins
J = Jenkins('http://localhost:8080')
print J.items()
j = J['foo']
j = J.get_job("foo")
b = j.get_last_build()
print b
mjn = b.get_master_job_name()
print(mjn)

EMPTY_JOB_CONFIG = '''\
<?xml version='1.0' encoding='UTF-8'?>
<project>
  <actions/>
  <description></description>
  <keepDependencies>false</keepDependencies>
  <properties/>
  <scm class="hudson.scm.NullSCM"/>
  <canRoam>true</canRoam>
  <disabled>false</disabled>
  <blockBuildWhenDownstreamBuilding>false</blockBuildWhenDownstreamBuilding>
  <blockBuildWhenUpstreamBuilding>false</blockBuildWhenUpstreamBuilding>
  <triggers class="vector"/>
  <concurrentBuild>false</concurrentBuild>
  <builders/>
  <publishers/>
  <buildWrappers/>
</project>
'''

new_job = J.create_job(name='foo_job', config=EMPTY_JOB_CONFIG)



"""
Queue module for jenkinsapi
"""

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import UnknownQueueItem
import logging

log = logging.getLogger(__name__)


class Queue(JenkinsBase):
    """
    Class that represents the Jenkins queue
    """

    def __init__(self, baseurl, jenkins_obj):
        """
        Init the Jenkins queue object
        :param baseurl: basic url for the queue
        :param jenkins_obj: ref to the jenkins obj
        """
        self.jenkins = jenkins_obj
        JenkinsBase.__init__(self, baseurl)

    def __str__(self):
        return self.baseurl

    def get_jenkins_obj(self):
        return self.jenkins

    def iteritems(self):
        for item in self._data['items']:
            yield item['id'], QueueItem(self.jenkins, **item)

    def iterkeys(self):
        for item in self._data['items']:
            yield item['id']

    def iterivalues(self):
        for item in self._data['items']:
            yield QueueItem(self.jenkins, **item)

    def keys(self):
        return list(self.iterkeys())

    def values(self):
        return list(self.itervalues())

    def __len__(self):
        return len(self._data['items'])

    def __getitem__(self, item_id):
        self_as_dict = dict(self.iteritems())
        if item_id in self_as_dict:
            return self_as_dict[item_id]
        else:
            raise UnknownQueueItem(item_id)

    def get_queue_items_for_job(self, job_name):
        if not job_name:
            return [QueueItem(self.jenkins, **item) for item in self._data['items']]
        else:
            return [QueueItem(self.jenkins, **item) for item in self._data['items']
                    if item['task']['name'] == job_name]

    def delete_item(self, queue_item):
        self.delete_item_by_id(queue_item.id)

    def delete_item_by_id(self, item_id):
        deleteurl = '%s/cancelItem?id=%s' % (self.baseurl, item_id)
        self.get_jenkins_obj().requester.post_url(deleteurl)


class QueueItem(object):
    """
    Flexible class to handle queue items. If the Jenkins API changes this support
    those changes
    """

    def __init__(self, jenkins, **kwargs):
        self.jenkins = jenkins
        self.__dict__.update(kwargs)

    def get_job(self):
        """
        Return the job associated with this queue item
        """
        return self.jenkins[self.task['name']]

    def get_parameters(self):
        """returns parameters of queue item"""
        actions = getattr(self, 'actions', [])
        for action in actions:
            if type(action) is dict and 'parameters' in action:
                parameters = action['parameters']
                return dict([(x['name'], x['value']) for x in parameters])
        return []

    def __repr__(self):
        return "<%s.%s %s>" % (self.__class__.__module__, self.__class__.__name__, str(self))

    def __str__(self):
        return "%s #%i" % (self.task['name'], self.id)



from jenkinsapi.api import search_artifacts

jenkinsurl = "http://localhost:8080/jenkins"
jobid = "test1"
artifact_ids = ["test1.txt", "test2.txt"]  # I need a build that contains all of these
result = search_artifacts(jenkinsurl, jobid, artifact_ids)
print((repr(result)))



from setuptools import setup
import os

PROJECT_ROOT, _ = os.path.split(__file__)
REVISION = '0.2.16'
PROJECT_NAME = 'JenkinsAPI'
PROJECT_AUTHORS = "Salim Fadhley, Ramon van Alteren, Ruslan Lutsenko"
PROJECT_EMAILS = 'salimfadhley@gmail.com, ramon@vanalteren.nl, ruslan.lutcenko@gmail.com'
PROJECT_URL = "https://github.com/salimfadhley/jenkinsapi"
SHORT_DESCRIPTION = 'A Python API for accessing resources on a Jenkins continuous-integration server.'

try:
    DESCRIPTION = open(os.path.join(PROJECT_ROOT, "README.rst")).read()
except IOError, _:
    DESCRIPTION = SHORT_DESCRIPTION

GLOBAL_ENTRY_POINTS = {
    "console_scripts": ["jenkins_invoke=jenkinsapi.command_line.jenkins_invoke:main"]
}

setup(
    name=PROJECT_NAME.lower(),
    version=REVISION,
    author=PROJECT_AUTHORS,
    author_email=PROJECT_EMAILS,
    packages=['jenkinsapi', 'jenkinsapi.utils', 'jenkinsapi.command_line', 'jenkinsapi_tests'],
    zip_safe=True,
    include_package_data=False,
    install_requires=['requests>=1.2.3', 'pytz>=2013b'],
    test_suite='nose.collector',
    tests_require=['mock', 'nose', 'coverage'],
    entry_points=GLOBAL_ENTRY_POINTS,
    url=PROJECT_URL,
    description=SHORT_DESCRIPTION,
    long_description=DESCRIPTION,
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Testing',
    ],
)



"""
Module for jenkinsapi Views
"""

import logging
import json
from jenkinsapi.view import View

log = logging.getLogger(__name__)


class Views(object):
    """
    An abstraction on a Jenkins object's views
    """
    LIST_VIEW = 'hudson.model.ListView'
    NESTED_VIEW = 'hudson.plugins.nested_view.NestedView'
    MY_VIEW = 'hudson.model.MyView'
    DASHBOARD_VIEW = 'hudson.plugins.view.dashboard.Dashboard'
    PIPELINE_VIEW = 'au.com.centrumsystems.hudson.plugin.buildpipeline.BuildPipelineView'

    def __init__(self, jenkins):
        self.jenkins = jenkins

    def __len__(self):
        return len(self.keys())

    def __delitem__(self, view_name):
        if view_name == 'All':
            raise ValueError('Cannot delete this view: %s' % view_name)

        if view_name in self:
            self[view_name].delete()
            self.jenkins.poll()

    def __setitem__(self, view_name, job_names_list):
        new_view = self.create(view_name)
        if isinstance(job_names_list, str):
            job_names_list = [job_names_list]
        for job_name in job_names_list:
            if not new_view.add_job(job_name):
                # Something wrong - delete view
                del self[new_view]
                raise TypeError('Job %s does not exist in Jenkins' % job_name)

    def __getitem__(self, view_name):
        for row in self.jenkins._data.get('views', []):
            if row['name'] == view_name:
                return View(row['url'], row['name'], self.jenkins)

    def iteritems(self):
        """
        Get the names & objects for all views
        """
        self.jenkins.poll()
        for row in self.jenkins._data.get('views', []):
            name = row['name']
            url = row['url']

            yield name, View(url, name, self.jenkins)

    def __contains__(self, view_name):
        """
        True if view_name is the name of a defined view
        """
        return view_name in self.keys()

    def iterkeys(self):
        """
        Get the names of all available views
        """
        for row in self.jenkins._data.get('views', []):
            yield row['name']

    def keys(self):
        """
        Return a list of the names of all views
        """
        return list(self.iterkeys())

    def create(self, view_name, view_type=LIST_VIEW):
        """
        Create a view
        :param view_name: name of new view, str
        :param person: Person name (to create personal view), str
        :return: new View obj or None if view was not created
        """
        log.info(msg='Creating "%s" view "%s"' % (view_type, view_name))
        #url = urlparse.urljoin(self.baseurl, "user/%s/my-views/" % person) if person else self.baseurl

        if view_name in self:
            log.warn(msg='View "%s" already exists' % view_name)
            return self[view_name]

        url = '%s/createView' % self.jenkins.baseurl
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        data = {
            "name": view_name,
            "mode": view_type,
            "Submit": "OK",
            "json": json.dumps({"name": view_name, "mode": view_type})
        }

        self.jenkins.requester.post_and_confirm_status(url, data=data, headers=headers)
        self.jenkins.poll()
        return self[view_name]



"""
Module for jenkinsapi views
"""
import urllib
import logging

from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.job import Job


log = logging.getLogger(__name__)


class View(JenkinsBase):
    """
    View class
    """

    def __init__(self, url, name, jenkins_obj):
        self.name = name
        self.jenkins_obj = jenkins_obj
        JenkinsBase.__init__(self, url)
        self.deleted = False

    def __str__(self):
        return self.name

    def __getitem__(self, job_name):
        assert isinstance(job_name, str)
        api_url = self.python_api_url(self.get_job_url(job_name))
        return Job(api_url, job_name, self.jenkins_obj)

    def __contains__(self, job_name):
        """
        True if view_name is the name of a defined view
        """
        return job_name in self.keys()

    def delete(self):
        """
        Remove this view object
        """
        url = "%s/doDelete" % self.baseurl
        self.jenkins_obj.requester.post_and_confirm_status(url, data='')
        self.jenkins_obj.poll()
        self.deleted = True

    def keys(self):
        return self.get_job_dict().keys()

    def iteritems(self):
        for name, url in self.get_job_dict().iteritems():
            api_url = self.python_api_url(url)
            yield name, Job(api_url, name, self.jenkins_obj)

    def values(self):
        return [a[1] for a in self.iteritems()]

    def items(self):
        return [a for a in self.iteritems()]

    def _get_jobs(self):
        if not 'jobs' in self._data:
            pass
        else:
            for viewdict in self._data["jobs"]:
                yield viewdict["name"], viewdict["url"]

    def get_job_dict(self):
        return dict(self._get_jobs())

    def __len__(self):
        return len(self.get_job_dict().keys())

    def get_job_url(self, str_job_name):
        try:
            job_dict = self.get_job_dict()
            return job_dict[str_job_name]
        except KeyError:
            #noinspection PyUnboundLocalVariable
            all_views = ", ".join(job_dict.keys())
            raise KeyError("Job %s is not known - available: %s" % (str_job_name, all_views))

    def get_jenkins_obj(self):
        return self.jenkins_obj

    def add_job(self, str_job_name, job=None):
        """
        Add job to a view

        :param str_job_name: name of the job to be added
        :param job: Job object to be added
        :return: True if job has been added, False if job already exists or
         job not known to Jenkins
        """
        if not job:
            if str_job_name in self.get_job_dict():
                log.warn(msg='Job %s is already in the view %s' %
                         (str_job_name, self.name))
                return False
            else:
                # Since this call can be made from nested view,
                # which doesn't have any jobs, we can miss existing job
                # Thus let's create top level Jenkins and ask him
                # http://jenkins:8080/view/CRT/view/CRT-FB/view/CRT-SCRT-1301/
                top_jenkins = self.get_jenkins_obj().get_jenkins_obj_from_url(
                    self.baseurl.split('view/')[0])
                if not top_jenkins.has_job(str_job_name):
                    log.error(msg='Job "%s" is not known to Jenkins' % str_job_name)
                    return False
                else:
                    job = top_jenkins.get_job(str_job_name)

        log.info(msg='Creating job %s in view %s' % (str_job_name, self.name))
        data = {
            "description": "",
            "statusFilter": "",
            "useincluderegex": "on",
            "includeRegex": "",
            "columns": [{"stapler-class": "hudson.views.StatusColumn",
                         "kind": "hudson.views.StatusColumn"},
                        {"stapler-class": "hudson.views.WeatherColumn",
                         "kind": "hudson.views.WeatherColumn"},
                        {"stapler-class": "hudson.views.JobColumn",
                         "kind": "hudson.views.JobColumn"},
                        {"stapler-class": "hudson.views.LastSuccessColumn",
                         "kind": "hudson.views.LastSuccessColumn"},
                        {"stapler-class": "hudson.views.LastFailureColumn",
                         "kind": "hudson.views.LastFailureColumn"},
                        {"stapler-class": "hudson.views.LastDurationColumn",
                         "kind": "hudson.views.LastDurationColumn"},
                        {"stapler-class": "hudson.views.BuildButtonColumn",
                         "kind": "hudson.views.BuildButtonColumn"}],
            "Submit": "OK",
        }
        data["name"] = self.name
        # Add existing jobs (if any)
        for job_name in self.get_job_dict().keys():
            data[job_name] = 'on'

        # Add new job
        data[job.name] = 'on'

        data['json'] = data.copy()
        data = urllib.urlencode(data)
        self.get_jenkins_obj().requester.post_and_confirm_status(
            '%s/configSubmit' % self.baseurl, data=data)
        self.poll()
        log.debug(msg='Job "%s" has been added to a view "%s"' %
                  (job.name, self.name))
        return True

    def _get_nested_views(self):
        for viewdict in self._data.get("views", []):
            yield viewdict["name"], viewdict["url"]

    def get_nested_view_dict(self):
        return dict(self._get_nested_views())

    @property
    def views(self):
        return self.get_jenkins_obj().get_jenkins_obj_from_url(self.baseurl).views



import mock
import types
import unittest

from jenkinsapi.custom_exceptions import JenkinsAPIException
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.executors import Executors
from jenkinsapi.executor import Executor


class TestExecutors(unittest.TestCase):

    DATAM = {
        'assignedLabels': [{}],
        'description': None,
        'jobs': [],
        'mode': 'NORMAL',
        'nodeDescription': 'the master Jenkins node',
        'nodeName': '',
        'numExecutors': 2,
        'overallLoad': {},
        'primaryView': {'name': 'All', 'url': 'http://localhost:8080/'},
        'quietingDown': False,
        'slaveAgentPort': 0,
        'unlabeledLoad': {},
        'useCrumbs': False,
        'useSecurity': False,
        'views': [
            {'name': 'All', 'url': 'http://localhost:8080/'},
            {'name': 'BigMoney', 'url': 'http://localhost:8080/view/BigMoney/'}
        ]
    }

    DATA0 = {
        "actions": [
        ],
        "displayName": "host0.host.com",
        "executors": [
            {},
            {},
            {},
            {},
            {},
            {},
            {},
            {}
        ],
        "icon": "computer.png",
        "idle": False,
        "jnlpAgent": True,
        "launchSupported": False,
        "loadStatistics": {

        },
        "manualLaunchAllowed": True,
        "monitorData": {
            "hudson.node_monitors.SwapSpaceMonitor": {
                "availablePhysicalMemory": 8462417920,
                "availableSwapSpace": 0,
                "totalPhysicalMemory": 75858042880,
                "totalSwapSpace": 0
            },
            "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
            "hudson.node_monitors.ResponseTimeMonitor": {
                "average": 2
            },
            "hudson.node_monitors.TemporarySpaceMonitor": {
                "path": "/tmp",
                "size": 430744551424
            },
            "hudson.node_monitors.DiskSpaceMonitor": {
                "path": "/data/jenkins",
                "size": 1214028627968
            },
            "hudson.node_monitors.ClockMonitor": {
                "diff": 1
            }
        },
        "numExecutors": 8,
        "offline": False,
        "offlineCause": None,
        "offlineCauseReason": "",
        "oneOffExecutors": [
            {},
            {}
        ],
        "temporarilyOffline": False
    }

    DATA1 = {
        "actions": [
        ],
        "displayName": "host1.host.com",
        "executors": [
            {},
            {}
        ],
        "icon": "computer.png",
        "idle": False,
        "jnlpAgent": True,
        "launchSupported": False,
        "loadStatistics": {
        },
        "manualLaunchAllowed": True,
        "monitorData": {
            "hudson.node_monitors.SwapSpaceMonitor": {
                "availablePhysicalMemory": 8462417920,
                "availableSwapSpace": 0,
                "totalPhysicalMemory": 75858042880,
                "totalSwapSpace": 0
            },
            "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
            "hudson.node_monitors.ResponseTimeMonitor": {
                "average": 2
            },
            "hudson.node_monitors.TemporarySpaceMonitor": {
                "path": "/tmp",
                "size": 430744551424
            },
            "hudson.node_monitors.DiskSpaceMonitor": {
                "path": "/data/jenkins",
                "size": 1214028627968
            },
            "hudson.node_monitors.ClockMonitor": {
                "diff": 1
            }
        },
        "numExecutors": 2,
        "offline": False,
        "offlineCause": None,
        "offlineCauseReason": "",
        "oneOffExecutors": [
            {},
            {}
        ],
        "temporarilyOffline": False
    }

    DATA2 = {
        "actions": [
        ],
        "displayName": "host2.host.com",
        "executors": [
            {},
            {},
            {},
            {}
        ],
        "icon": "computer.png",
        "idle": False,
        "jnlpAgent": True,
        "launchSupported": False,
        "loadStatistics": {
        },
        "manualLaunchAllowed": True,
        "monitorData": {
            "hudson.node_monitors.SwapSpaceMonitor": {
                "availablePhysicalMemory": 8462417920,
                "availableSwapSpace": 0,
                "totalPhysicalMemory": 75858042880,
                "totalSwapSpace": 0
            },
            "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
            "hudson.node_monitors.ResponseTimeMonitor": {
                "average": 2
            },
            "hudson.node_monitors.TemporarySpaceMonitor": {
                "path": "/tmp",
                "size": 430744551424
            },
            "hudson.node_monitors.DiskSpaceMonitor": {
                "path": "/data/jenkins",
                "size": 1214028627968
            },
            "hudson.node_monitors.ClockMonitor": {
                "diff": 1
            }
        },
        "numExecutors": 4,
        "offline": False,
        "offlineCause": None,
        "offlineCauseReason": "",
        "oneOffExecutors": [
            {},
            {}
        ],
        "temporarilyOffline": False
    }

    DATA3 = {
        "actions": [
        ],
        "displayName": "host3.host.com",
        "executors": [
            {}
        ],
        "icon": "computer.png",
        "idle": False,
        "jnlpAgent": True,
        "launchSupported": False,
        "loadStatistics": {},
        "manualLaunchAllowed": True,
        "monitorData": {
            "hudson.node_monitors.SwapSpaceMonitor": {
                "availablePhysicalMemory": 8462417920,
                "availableSwapSpace": 0,
                "totalPhysicalMemory": 75858042880,
                "totalSwapSpace": 0
            },
            "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
            "hudson.node_monitors.ResponseTimeMonitor": {
                "average": 2
            },
            "hudson.node_monitors.TemporarySpaceMonitor": {
                "path": "/tmp",
                "size": 430744551424
            },
            "hudson.node_monitors.DiskSpaceMonitor": {
                "path": "/data/jenkins",
                "size": 1214028627968
            },
            "hudson.node_monitors.ClockMonitor": {
                "diff": 1
            }
        },
        "numExecutors": 1,
        "offline": False,
        "offlineCause": None,
        "offlineCauseReason": "",
        "oneOffExecutors": [
            {},
            {}
        ],
        "temporarilyOffline": False
    }

    EXEC0 = {
        "currentExecutable": {
            "number": 4168,
            "url": "http://localhost:8080/job/testjob/4168/"
        },
        "currentWorkUnit": {},
        "idle": False,
        "likelyStuck": False,
        "number": 0,
        "progress": 48
    }

    EXEC1 = {
        "currentExecutable": None,
        "currentWorkUnit": None,
        "idle": True,
        "likelyStuck": False,
        "number": 0,
        "progress": -1
    }

    @mock.patch.object(Jenkins, '_poll')
    def setUp(self, _poll_jenkins):
        _poll_jenkins.return_value = self.DATAM
        self.J = Jenkins('http://localhost:8080')

    def testRepr(self):
        # Can we produce a repr string for this object
        assert repr(self.J)

    def testCheckURL(self):
        self.assertEquals(self.J.baseurl, 'http://localhost:8080')

    @mock.patch.object(Executors, '_poll')
    @mock.patch.object(Executor, '_poll')
    def testGetExecutors(self, _poll_executor, _poll_executors):
        _poll_executor.return_value = self.EXEC0
        _poll_executors.return_value = self.DATA3
        exec_info = self.J.get_executors(self.DATA3['displayName'])

        self.assertIsInstance(exec_info, object)
        self.assertIsInstance(repr(exec_info), str)

        for e in exec_info:
            self.assertEquals(e.get_progress(), 48, 'Should return 48 %')

    @mock.patch.object(Executors, '_poll')
    @mock.patch.object(Executor, '_poll')
    def testis_idle(self, _poll_executor, _poll_executors):
        _poll_executor.return_value = self.EXEC1
        _poll_executors.return_value = self.DATA3
        exec_info = self.J.get_executors('host3.host.com')

        self.assertIsInstance(exec_info, object)
        for e in exec_info:
            self.assertEquals(e.get_progress(), -1, 'Should return 48 %')
            self.assertEquals(e.is_idle(), True, 'Should return True')
            self.assertEquals(
                repr(e),
                '<jenkinsapi.executor.Executor host3.host.com 0>'
            )

    @mock.patch.object(Executor, '_poll')
    def test_likely_stuck(self, _poll_executor):
        _poll_executor.return_value = self.EXEC0
        baseurl = 'http://localhost:8080/computer/host0.host.com/executors/0'
        nodename = 'host0.host.com'
        single_executer = Executor(baseurl, nodename, self.J, '0')
        self.assertEquals(single_executer.likely_stuck(), False)

    @mock.patch.object(Executor, '_poll')
    def test_get_current_executable(self, _poll_executor):
        _poll_executor.return_value = self.EXEC0
        baseurl = 'http://localhost:8080/computer/host0.host.com/executors/0'
        nodename = 'host0.host.com'
        single_executer = Executor(baseurl, nodename, self.J, '0')
        self.assertEquals(
            single_executer.get_current_executable()['number'],
            4168
        )
        self.assertEquals(
            single_executer.get_current_executable()['url'],
            'http://localhost:8080/job/testjob/4168/'
        )


if __name__ == '__main__':
    unittest.main()



"""
A jenkins build represents a single execution of a Jenkins Job.

Builds can be thought of as the second level of the jenkins heirarchy
beneath Jobs. Builds can have state, such as whether they are running or
not. They can also have outcomes, such as wether they passed or failed.

Build objects can be associated with Results and Artifacts.g
"""

import time
import pytz
import logging
import warnings
import datetime
from time import sleep
from jenkinsapi import config
from jenkinsapi.artifact import Artifact
from jenkinsapi.result_set import ResultSet
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.constants import STATUS_SUCCESS
from jenkinsapi.custom_exceptions import NoResults


log = logging.getLogger(__name__)


class Build(JenkinsBase):
    """
    Represents a jenkins build, executed in context of a job.
    """

    STR_TOTALCOUNT = "totalCount"
    STR_TPL_NOTESTS_ERR = "%s has status %s, and does not have any test results"

    def __init__(self, url, buildno, job):
        assert type(buildno) == int
        self.buildno = buildno
        self.job = job
        JenkinsBase.__init__(self, url)

    def _poll(self):
        #For build's we need more information for downstream and upstream builds
        #so we override the poll to get at the extra data for build objects
        url = self.python_api_url(self.baseurl) + '?depth=2'
        return self.get_data(url)

    def __str__(self):
        return self._data['fullDisplayName']

    @property
    def name(self):
        return str(self)

    def get_number(self):
        return self._data["number"]

    def get_status(self):
        return self._data["result"]

    def get_revision(self):
        vcs = self._data['changeSet']['kind'] or 'git'
        return getattr(self, '_get_%s_rev' % vcs, lambda: None)()

    def get_revision_branch(self):
        vcs = self._data['changeSet']['kind'] or 'git'
        return getattr(self, '_get_%s_rev_branch' % vcs, lambda: None)()

    def _get_svn_rev(self):
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        maxRevision = 0
        for repoPathSet in self._data["changeSet"]["revisions"]:
            maxRevision = max(repoPathSet["revision"], maxRevision)
        return maxRevision

    def _get_git_rev(self):
        # Sometimes we have None as part of actions. Filter those actions
        # which have lastBuiltRevision in them
        _actions = [x for x in self._data['actions']
                    if x and "lastBuiltRevision" in x]

        return _actions[0]["lastBuiltRevision"]["SHA1"]

    def _get_hg_rev(self):
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        return [x['mercurialNodeName'] for x in self._data['actions'] if 'mercurialNodeName' in x][0]

    def _get_svn_rev_branch(self):
        raise NotImplementedError('_get_svn_rev_branch is not yet implemented')

    def _get_git_rev_branch(self):
        # Sometimes we have None as part of actions. Filter those actions
        # which have lastBuiltRevision in them
        _actions = [x for x in self._data['actions']
                    if x and "lastBuiltRevision" in x]

        return _actions[0]["lastBuiltRevision"]["branch"]

    def _get_hg_rev_branch(self):
        raise NotImplementedError('_get_hg_rev_branch is not yet implemented')

    def get_duration(self):
        return datetime.timedelta(milliseconds=self._data["duration"])

    def get_artifacts(self):
        for afinfo in self._data["artifacts"]:
            url = "%s/artifact/%s" % (self.baseurl, afinfo["relativePath"])
            af = Artifact(afinfo["fileName"], url, self)
            yield af

    def get_artifact_dict(self):
        return dict(
            (af.filename, af) for af in self.get_artifacts()
        )

    def get_upstream_job_name(self):
        """
        Get the upstream job name if it exist, None otherwise
        :return: String or None
        """
        try:
            return self.get_actions()['causes'][0]['upstreamProject']
        except KeyError:
            return None

    def get_upstream_job(self):
        """
        Get the upstream job object if it exist, None otherwise
        :return: Job or None
        """
        if self.get_upstream_job_name():
            return self.get_jenkins_obj().get_job(self.get_upstream_job_name())
        else:
            return None

    def get_upstream_build_number(self):
        """
        Get the upstream build number if it exist, None otherwise
        :return: int or None
        """
        try:
            return int(self.get_actions()['causes'][0]['upstreamBuild'])
        except KeyError:
            return None

    def get_upstream_build(self):
        """
        Get the upstream build if it exist, None otherwise
        :return Build or None
        """
        upstream_job = self.get_upstream_job()
        if upstream_job:
            return upstream_job.get_build(self.get_upstream_build_number())
        else:
            return None

    def get_master_job_name(self):
        """
        Get the master job name if it exist, None otherwise
        :return: String or None
        """
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        try:
            return self.get_actions()['parameters'][0]['value']
        except KeyError:
            return None

    def get_master_job(self):
        """
        Get the master job object if it exist, None otherwise
        :return: Job or None
        """
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        if self.get_master_job_name():
            return self.get_jenkins_obj().get_job(self.get_master_job_name())
        else:
            return None

    def get_master_build_number(self):
        """
        Get the master build number if it exist, None otherwise
        :return: int or None
        """
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        try:
            return int(self.get_actions()['parameters'][1]['value'])
        except KeyError:
            return None

    def get_master_build(self):
        """
        Get the master build if it exist, None otherwise
        :return Build or None
        """
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        master_job = self.get_master_job()
        if master_job:
            return master_job.get_build(self.get_master_build_number())
        else:
            return None

    def get_downstream_jobs(self):
        """
        Get the downstream jobs for this build
        :return List of jobs or None
        """
        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
        downstream_jobs = []
        try:
            for job_name in self.get_downstream_job_names():
                downstream_jobs.append(self.get_jenkins_obj().get_job(job_name))
            return downstream_jobs
        except (IndexError, KeyError):
            return []

    def get_downstream_job_names(self):
        """
        Get the downstream job names for this build
        :return List of string or None
        """
# <<<<<<< HEAD
#         downstream_jobs_names = self.job.get_downstream_job_names()
#         fingerprint_data = self.get_data("%s?depth=2&tree=fingerprint[usage[name]]" \
#                                            % self.python_api_url(self.baseurl))
#         try:
#             fingerprints = fingerprint_data['fingerprint'][0]
#             return [
#                 f['name']
#                 for f in fingerprints['usage']
#                 if f['name'] in downstream_jobs_names
#             ]
# =======
        downstream_job_names = self.job.get_downstream_job_names()
        downstream_names = []
        try:
            fingerprints = self._data["fingerprint"]
            for fingerprint in fingerprints:
                for job_usage in fingerprint['usage']:
                    if job_usage['name'] in downstream_job_names:
                        downstream_names.append(job_usage['name'])
            return downstream_names
# >>>>>>> unstable
        except (IndexError, KeyError):
            return []

    def get_downstream_builds(self):
        """
        Get the downstream builds for this build
        :return List of Build or None
        """
# <<<<<<< HEAD
#         downstream_jobs_names = set(self.job.get_downstream_job_names())
#         msg = "%s?depth=2&tree=fingerprint[usage[name,ranges[ranges[end,start]]]]"
#         fingerprint_data = self.get_data(msg % self.python_api_url(self.baseurl))
#         try:
#             fingerprints = fingerprint_data['fingerprint'][0]
#             return [
#                 self.get_jenkins_obj().get_job(f['name']).get_build(f['ranges']['ranges'][0]['start'])
#                 for f in fingerprints['usage']
#                 if f['name'] in downstream_jobs_names
#             ]
# =======
        downstream_job_names = self.get_downstream_job_names()
        downstream_builds = []
        try:
            fingerprints = self._data["fingerprint"]
            for fingerprint in fingerprints:
                for job_usage in fingerprint['usage']:
                    if job_usage['name'] in downstream_job_names:
                        job = self.get_jenkins_obj().get_job(job_usage['name'])
                        for job_range in job_usage['ranges']['ranges']:
                            for build_id in range(job_range['start'],
                                                  job_range['end']):
                                downstream_builds.append(job.get_build(build_id))
            return downstream_builds
# >>>>>>> unstable
        except (IndexError, KeyError):
            return []

    def get_matrix_runs(self):
        """
        For a matrix job, get the individual builds for each
        matrix configuration
        :return: Generator of Build
        """
        if "runs" in self._data:
            for rinfo in self._data["runs"]:
                yield Build(rinfo["url"], rinfo["number"], self.job)

    def is_running(self):
        """
        Return a bool if running.
        """
        self.poll()
        return self._data["building"]

    def block(self):
        while self.is_running():
            time.sleep(1)

    def is_good(self):
        """
        Return a bool, true if the build was good.
        If the build is still running, return False.
        """
        return (not self.is_running()) and self._data["result"] == STATUS_SUCCESS

    def block_until_complete(self, delay=15):
        assert isinstance(delay, int)
        count = 0
        while self.is_running():
            total_wait = delay * count
            log.info(msg="Waited %is for %s #%s to complete" % (total_wait, self.job.name, self.name))
            sleep(delay)
            count += 1

    def get_jenkins_obj(self):
        return self.job.get_jenkins_obj()

    def get_result_url(self):
        """
        Return the URL for the object which provides the job's result summary.
        """
        url_tpl = r"%stestReport/%s"
        return url_tpl % (self._data["url"], config.JENKINS_API)

    def get_resultset(self):
        """
        Obtain detailed results for this build.
        """
        result_url = self.get_result_url()
        if self.STR_TOTALCOUNT not in self.get_actions():
            raise NoResults("%s does not have any published results" % str(self))
        buildstatus = self.get_status()
        if not self.get_actions()[self.STR_TOTALCOUNT]:
            raise NoResults(self.STR_TPL_NOTESTS_ERR % (str(self), buildstatus))
        obj_results = ResultSet(result_url, build=self)
        return obj_results

    def has_resultset(self):
        """
        Return a boolean, true if a result set is available. false if not.
        """
        return self.STR_TOTALCOUNT in self.get_actions()

    def get_actions(self):
        all_actions = {}
        for dct_action in self._data["actions"]:
            if dct_action is None:
                continue
            all_actions.update(dct_action)
        return all_actions

    def get_timestamp(self):
        '''
        Returns build timestamp in UTC
        '''
        # Java timestamps are given in miliseconds since the epoch start!
        naive_timestamp = datetime.datetime(*time.gmtime(self._data['timestamp'] / 1000.0)[:6])
        return pytz.utc.localize(naive_timestamp)

    def get_console(self):
        """
        Return the current state of the text console.
        """
        url = "%s/consoleText" % self.baseurl
        return self.job.jenkins.requester.get_url(url).content

    def stop(self):
        """
        Stops the build execution if it's running
        :return boolean True if succeded False otherwise or the build is not running
        """
        if self.is_running():
            url = "%s/stop" % self.baseurl
            self.job.jenkins.requester.post_and_confirm_status(url, data='')
            return True
        return False



"""
An example of how to use JenkinsAPI to fetch the config XML of a job.
"""
from jenkinsapi.jenkins import Jenkins
J = Jenkins('http://localhost:8080')
jobName = 'create_fwrgmkbbzk'

config = J[jobName].get_config()

print config



from jenkinsapi.api import search_artifact_by_regexp
import re

jenkinsurl = "http://localhost:8080/jenkins"
jobid = "test1"
artifact_regexp = re.compile("test1\.txt")  # A file name I want.
result = search_artifact_by_regexp(jenkinsurl, jobid, artifact_regexp)
print((repr(result)))



"""
Module for jenkinsapi Jenkins object
"""


import json
import urllib
import logging
import urlparse

from jenkinsapi import config
from jenkinsapi.executors import Executors
from jenkinsapi.job import Job
from jenkinsapi.jobs import Jobs
from jenkinsapi.node import Node
from jenkinsapi.view import View
from jenkinsapi.nodes import Nodes
from jenkinsapi.plugins import Plugins
from jenkinsapi.views import Views
from jenkinsapi.queue import Queue
from jenkinsapi.fingerprint import Fingerprint
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.utils.requester import Requester
from jenkinsapi.custom_exceptions import UnknownJob

log = logging.getLogger(__name__)


class Jenkins(JenkinsBase):
    """
    Represents a jenkins environment.
    """
    def __init__(self, baseurl, username=None, password=None, requester=None):
        """
        :param baseurl: baseurl for jenkins instance including port, str
        :param username: username for jenkins auth, str
        :param password: password for jenkins auth, str
        :return: a Jenkins obj
        """
        self.username = username
        self.password = password
        self.requester = requester or Requester(username, password, baseurl=baseurl)
        JenkinsBase.__init__(self, baseurl)

    def _clone(self):
        return Jenkins(self.baseurl, username=self.username,
                       password=self.password, requester=self.requester)

    def base_server_url(self):
        if config.JENKINS_API in self.baseurl:
            return self.baseurl[:-(len(config.JENKINS_API))]
        else:
            return self.baseurl

    def validate_fingerprint(self, id_):
        obj_fingerprint = Fingerprint(self.baseurl, id_, jenkins_obj=self)
        obj_fingerprint.validate()
        log.info(msg="Jenkins says %s is valid" % id_)

    # def reload(self):
    #     '''Try and reload the configuration from disk'''
    #     self.requester.get_url("%(baseurl)s/reload" % self.__dict__)

    def get_artifact_data(self, id_):
        obj_fingerprint = Fingerprint(self.baseurl, id_, jenkins_obj=self)
        obj_fingerprint.validate()
        return obj_fingerprint.get_info()

    def validate_fingerprint_for_build(self, digest, filename, job, build):
        obj_fingerprint = Fingerprint(self.baseurl, digest, jenkins_obj=self)
        return obj_fingerprint.validate_for_build(filename, job, build)

    def get_jenkins_obj(self):
        return self

    def get_jenkins_obj_from_url(self, url):
        return Jenkins(url, self.username, self.password, self.requester)

    def get_create_url(self):
        # This only ever needs to work on the base object
        return '%s/createItem' % self.baseurl

    def get_nodes_url(self):
        # This only ever needs to work on the base object
        return '%s/computer' % self.baseurl

    @property
    def jobs(self):
        return Jobs(self)

    def get_jobs(self):
        """
        Fetch all the build-names on this Jenkins server.
        """
        for info in self._data["jobs"]:
            yield info["name"], \
                Job(info["url"], info["name"], jenkins_obj=self)

    def get_jobs_info(self):
        """
        Get the jobs information
        :return url, name
        """
        for info in self._data["jobs"]:
            yield info["url"], info["name"]

    def get_job(self, jobname):
        """
        Get a job by name
        :param jobname: name of the job, str
        :return: Job obj
        """
        return self.jobs[jobname]

    def has_job(self, jobname):
        """
        Does a job by the name specified exist
        :param jobname: string
        :return: boolean
        """
        return jobname in self.jobs

    def create_job(self, jobname, config_):
        """
        Create a job
        :param jobname: name of new job, str
        :param config: configuration of new job, xml
        :return: new Job obj
        """
        return self.jobs.create(jobname, config_)

    def copy_job(self, jobname, newjobname):
        return self.jobs.copy(jobname, newjobname)

    def build_job(self, jobname, params=None):
        """
        Invoke a build by job name
        :param jobname: name of exist job, str
        :param params: the job params, dict
        :return: none
        """
        self[jobname].invoke(build_params=params or {})

    def delete_job(self, jobname):
        """
        Delete a job by name
        :param jobname: name of a exist job, str
        :return: new jenkins_obj
        """
        del self.jobs[jobname]

    def rename_job(self, jobname, newjobname):
        """
        Rename a job
        :param jobname: name of a exist job, str
        :param newjobname: name of new job, str
        :return: new Job obj
        """
        return self.jobs.rename(jobname, newjobname)

    def iterkeys(self):
        for info in self._data["jobs"]:
            yield info["name"]

    def iteritems(self):
        """
        :param return: An iterator of pairs.
            Each pair will be (job name, Job object)
        """
        return self.get_jobs()

    def items(self):
        """
        :param return: A list of pairs. Each pair will be (job name, Job object)
        """
        return list(self.get_jobs())

    def keys(self):
        return [a for a in self.iterkeys()]

    # This is a function alias we retain for historical compatibility
    get_jobs_list = keys

    def __str__(self):
        return "Jenkins server at %s" % self.baseurl

    @property
    def views(self):
        return Views(self)

    def get_view_by_url(self, str_view_url):
        #for nested view
        str_view_name = str_view_url.split('/view/')[-1].replace('/', '')
        return View(str_view_url, str_view_name, jenkins_obj=self)

    def delete_view_by_url(self, str_url):
        url = "%s/doDelete" % str_url
        self.requester.post_and_confirm_status(url, data='')
        self.poll()
        return self

    def __getitem__(self, jobname):
        """
        Get a job by name
        :param jobname: name of job, str
        :return: Job obj
        """
        for info in self._data["jobs"]:
            if info["name"] == jobname:
                return Job(info["url"], info["name"], jenkins_obj=self)
        raise UnknownJob(jobname)

    def __len__(self):
        return len(self._data["jobs"])

    def __contains__(self, jobname):
        """
        Does a job by the name specified exist
        :param jobname: string
        :return: boolean
        """
        return jobname in self.jobs

    def get_node(self, nodename):
        """Get a node object for a specific node"""
        return self.get_nodes()[nodename]

    def get_node_url(self, nodename=""):
        """Return the url for nodes"""
        url = urlparse.urljoin(self.base_server_url(), 'computer/%s' % urllib.quote(nodename))
        return url

    def get_queue_url(self):
        url = "%s/%s" % (self.base_server_url(), 'queue')
        return url

    def get_queue(self):
        queue_url = self.get_queue_url()
        return Queue(queue_url, self)

    def get_nodes(self):
        url = self.get_nodes_url()
        return Nodes(url, self)

    def has_node(self, nodename):
        """
        Does a node by the name specified exist
        :param nodename: string, hostname
        :return: boolean
        """
        self.poll()
        return nodename in self.get_nodes()

    def delete_node(self, nodename):
        """
        Remove a node from the managed slave list
        Please note that you cannot remove the master node

        :param nodename: string holding a hostname
        :return: None
        """
        assert self.has_node(nodename), "This node: %s is not registered as a slave" % nodename
        assert nodename != "master", "you cannot delete the master node"
        url = "%s/doDelete" % self.get_node_url(nodename)
        self.requester.get_and_confirm_status(url)

    def create_node(self, name, num_executors=2, node_description=None,
                    remote_fs='/var/lib/jenkins', labels=None, exclusive=False):
        """
        Create a new slave node by name.

        :param name: fqdn of slave, str
        :param num_executors: number of executors, int
        :param node_description: a freetext field describing the node
        :param remote_fs: jenkins path, str
        :param labels: labels to associate with slave, str
        :param exclusive: tied to specific job, boolean
        :return: node obj
        """
        NODE_TYPE = 'hudson.slaves.DumbSlave$DescriptorImpl'
        MODE = 'NORMAL'
        if self.has_node(name):
            return Node(nodename=name, baseurl=self.get_node_url(nodename=name), jenkins_obj=self)
        if exclusive:
            MODE = 'EXCLUSIVE'
        params = {
            'name': name,
            'type': NODE_TYPE,
            'json': json.dumps({
                'name': name,
                'nodeDescription': node_description,
                'numExecutors': num_executors,
                'remoteFS': remote_fs,
                'labelString': labels,
                'mode': MODE,
                'type': NODE_TYPE,
                'retentionStrategy': {'stapler-class': 'hudson.slaves.RetentionStrategy$Always'},
                'nodeProperties': {'stapler-class-bag': 'true'},
                'launcher': {'stapler-class': 'hudson.slaves.JNLPLauncher'}
            })
        }
        url = self.get_node_url() + "doCreateItem?%s" % urllib.urlencode(params)
        self.requester.get_and_confirm_status(url)

        return Node(nodename=name, baseurl=self.get_node_url(nodename=name), jenkins_obj=self)

    def get_plugins_url(self):
        # This only ever needs to work on the base object
        return '%s/pluginManager/api/python?depth=1' % self.baseurl

    def get_plugins(self):
        url = self.get_plugins_url()
        return Plugins(url, self)

    def has_plugin(self, plugin_name):
        return plugin_name in self.get_plugins()

    def get_executors(self, nodename):
        url = '%s/computer/%s' % (self.baseurl, nodename)
        return Executors(url, nodename, self)

    @property
    def version(self):
        """
        Return version number of Jenkins
        """
        response = self.requester.get_and_confirm_status(self.baseurl)
        version_key = 'X-Jenkins'
        return response.headers.get(version_key, '0.0')



import mock
import unittest

from jenkinsapi.result_set import ResultSet
from jenkinsapi.result import Result


class TestResultSet(unittest.TestCase):

    DATA = {'duration': 0.0,
            'failCount': 2,
            'passCount': 0,
            'skipCount': 0,
            'suites': [{'cases': [{'age': 1,
                        'className': '<nose.suite.ContextSuite context=jenkinsapi_tests',
                        'duration': 0.0,
                        'errorDetails': 'Timeout error occured while waiting for Jenkins start.',
                        'errorStackTrace': 'Traceback (most recent call last):\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 208, in run\n    self.setUp()\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 291, in setUp\n    self.setupContext(ancestor)\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 314, in setupContext\n    try_run(context, names)\n  File "/usr/lib/python2.7/dist-packages/nose/util.py", line 478, in try_run\n    return func()\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 54, in setUpPackage\n    launcher = JenkinsLauncher(update_war=True, launch=True)\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 20, in __init__\n    self.launch()\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 41, in launch\n    raise Timeout(\'Timeout error occured while waiting for Jenkins start.\')\nTimeout: Timeout error occured while waiting for Jenkins start.\n',
                        'failedSince': 88,
                        'name': 'systests>:setup',
                        'skipped': False,
                        'status': 'FAILED',
                        'stderr': None,
                        'stdout': None},
                       {'age': 1,
                        'className': 'nose.failure.Failure',
                        'duration': 0.0,
                        'errorDetails': 'No module named mock',
                        'errorStackTrace': 'Traceback (most recent call last):\n  File "/usr/lib/python2.7/unittest/case.py", line 332, in run\n    testMethod()\n  File "/usr/lib/python2.7/dist-packages/nose/loader.py", line 390, in loadTestsFromName\n    addr.filename, addr.module)\n  File "/usr/lib/python2.7/dist-packages/nose/importer.py", line 39, in importFromPath\n    return self.importFromDir(dir_path, fqname)\n  File "/usr/lib/python2.7/dist-packages/nose/importer.py", line 86, in importFromDir\n    mod = load_module(part_fqname, fh, filename, desc)\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/unittests/test_build.py", line 1, in <module>\n    import mock\nImportError: No module named mock\n',
                        'failedSince': 88,
                        'name': 'runTest',
                        'skipped': False,
                        'status': 'FAILED',
                        'stderr': None,
                        'stdout': None}],
                        'duration': 0.0,
                        'id': None,
                        'name': 'nosetests',
                        'stderr': None,
                        'stdout': None,
                        'timestamp': None}]}

    @mock.patch.object(ResultSet, '_poll')
    def setUp(self, _poll):
        _poll.return_value = self.DATA

        # def __init__(self, url, build ):

        self.b = mock.MagicMock()  # Build object
        self.b.__str__.return_value = 'FooBuild'
        self.rs = ResultSet('http://', self.b)

    def testRepr(self):
        # Can we produce a repr string for this object
        repr(self.rs)

    def testName(self):
        with self.assertRaises(AttributeError):
            self.rs.id()

        self.assertEquals(self.rs.name, 'Test Result for FooBuild')

    def testBuildComponents(self):
        self.assertTrue(self.rs.items())
        for k, v in self.rs.items():
            self.assertIsInstance(k, str)
            self.assertIsInstance(v, Result)
            self.assertIsInstance(v.identifier(), str)



import mock
import unittest
from jenkinsapi.build import Build


class test_build(unittest.TestCase):

    DATA = {
        'actions': [
            {
                'causes': [{'shortDescription': 'Started by an SCM change'}]
            },
            {},
            {
                'buildsByBranchName': {
                    'origin/HEAD': {
                        'buildNumber': 2,
                        'buildResult': None,
                        'revision': {
                            'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                            'branch': [
                                {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2', 'name': 'origin/HEAD'},
                                {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2', 'name': 'origin/master'}
                            ]
                        }
                    },
                    'origin/master': {
                        'buildNumber': 2,
                        'buildResult': None,
                        'revision': {
                            'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2',
                            'branch': [
                                {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2', 'name': 'origin/HEAD'},
                                {'SHA1': 'd2a5d435fa2df3bff572bd06e43c86544749c5d2', 'name': 'origin/master'}
                            ]
                        }
                    },
                    'origin/python_3_compatibility': {
                        'buildNumber': 1,
                        'buildResult': None,
                        'revision': {
                            'SHA1': 'c9d1c96bc926ff63a5209c51b3ed537e62ea50e6',
                            'branch': [
                                {'SHA1': 'c9d1c96bc926ff63a5209c51b3ed537e62ea50e6', 'name': 'origin/python_3_compatibility'}
                            ]
                        }
                    },
                    'origin/unstable': {
                        'buildNumber': 3,
                        'buildResult': None,
                        'revision': {
                            'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
                            'branch': [
                                {'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702', 'name': 'origin/unstable'}
                            ]
                        }
                    }
                },
                'lastBuiltRevision': {
                    'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702',
                    'branch': [
                        {'SHA1': '7def9ed6e92580f37d00e4980c36c4d36e68f702', 'name': 'origin/unstable'}
                    ]
                },
                'remoteUrls': ['https://github.com/salimfadhley/jenkinsapi.git'],
                'scmName': ''
            },
            {},
            {}
        ],
        'artifacts': [],
        'building': False,
        'builtOn': '',
        'changeSet': {'items': [], 'kind': 'git'},
        'culprits': [],
        'description': None,
        'duration': 1051,
        'estimatedDuration': 2260,
        'executor': None,
        'fullDisplayName': 'git_yssrtigfds #3',
        'id': '2013-06-30_01-54-35',
        'keepLog': False,
        'number': 3,
        'result': 'SUCCESS',
        'timestamp': 1372553675652,
        'url': 'http://localhost:8080/job/git_yssrtigfds/3/'
    }

    @mock.patch.object(Build, '_poll')
    def setUp(self, _poll):
        _poll.return_value = self.DATA
        self.j = mock.MagicMock()  # Job
        self.j.name = 'FooJob'

        self.b = Build('http://', 97, self.j)

    def test_git_scm(self):
        """
        Can we extract git build revision data from a build object?
        """
        self.assertIsInstance(self.b.get_revision(), basestring)
        self.assertEquals(self.b.get_revision(),
                          '7def9ed6e92580f37d00e4980c36c4d36e68f702')

    def test_git_revision_branch(self):
        """
        Can we extract git build branch from a build object?
        """
        self.assertIsInstance(self.b.get_revision_branch(), list)
        self.assertEquals(len(self.b.get_revision_branch()), 1)
        self.assertIsInstance(self.b.get_revision_branch()[0], dict)
        self.assertEquals(self.b.get_revision_branch()[0]['SHA1'],
                          '7def9ed6e92580f37d00e4980c36c4d36e68f702')
        self.assertEquals(self.b.get_revision_branch()[0]['name'],
                          'origin/unstable')

if __name__ == '__main__':
    unittest.main()



# '''
# System tests for `jenkinsapi.jenkins` module.
# '''
# import unittest
# from jenkinsapi_tests.systests.base import BaseSystemTest
# from jenkinsapi_tests.test_utils.random_strings import random_string
# from jenkinsapi_tests.systests.job_configs import SCM_GIT_JOB

# # Maybe have a base class for all SCM test activites?
# class TestSCMGit(BaseSystemTest):
#     # Maybe it makes sense to move plugin dependencies outside the code.
#     # Have a config to dependencies mapping from the launcher can use to install plugins.
#     def test_get_revision(self):
#         job_name = 'git_%s' % random_string()
#         job = self.jenkins.create_job(job_name, SCM_GIT_JOB)
#         ii = job.invoke()
#         ii.block(until='completed')
#         self.assertFalse(ii.is_running())
#         b = ii.get_build()
#         self.assertIsInstance(b.get_revision(), basestring)

# if __name__ == '__main__':
#     unittest.main()



import mock
import unittest

import requests
from jenkinsapi.jenkins import Requester
from jenkinsapi.custom_exceptions import JenkinsAPIException


class TestQueue(unittest.TestCase):

    def test_get_request_dict_auth(self):
        req = Requester('foo', 'bar')

        req_return = req.get_request_dict(
            params={},
            data=None,
            headers=None
        )
        self.assertTrue(isinstance(req_return, dict))
        self.assertTrue(req_return.get('auth'))
        self.assertTrue(req_return['auth'] == ('foo', 'bar'))

    def test_get_request_dict_wrong_params(self):
        req = Requester('foo', 'bar')

        with self.assertRaises(AssertionError) as na:
            req.get_request_dict(
                params='wrong',
                data=None,
                headers=None
            )
        self.assertTrue(na.exception.message == "Params must be a dict, got 'wrong'")

    def test_get_request_dict_correct_params(self):
        req = Requester('foo', 'bar')

        req_return = req.get_request_dict(
            params={'param': 'value'},
            data=None,
            headers=None
        )

        self.assertTrue(isinstance(req_return, dict))
        self.assertTrue(req_return.get('params'))
        self.assertTrue(req_return['params'] == {'param': 'value'})

    def test_get_request_dict_wrong_headers(self):
        req = Requester('foo', 'bar')

        with self.assertRaises(AssertionError) as na:
            req.get_request_dict(
                params={},
                data=None,
                headers='wrong'
            )
        self.assertTrue(na.exception.message == "headers must be a dict, got 'wrong'")

    def test_get_request_dict_correct_headers(self):
        req = Requester('foo', 'bar')

        req_return = req.get_request_dict(
            params={'param': 'value'},
            data=None,
            headers={'header': 'value'}
        )

        self.assertTrue(isinstance(req_return, dict))
        self.assertTrue(req_return.get('headers'))
        self.assertTrue(req_return['headers'] == {'header': 'value'})

    def test_get_request_dict_data_passed(self):
        req = Requester('foo', 'bar')

        req_return = req.get_request_dict(
            params={'param': 'value'},
            data='some data',
            headers={'header': 'value'}
        )

        self.assertTrue(isinstance(req_return, dict))
        print req_return.get('data')
        self.assertTrue(req_return.get('data'))
        self.assertTrue(req_return['data'] == 'some data')

    def test_get_request_dict_data_not_passed(self):
        req = Requester('foo', 'bar')

        req_return = req.get_request_dict(
            params={'param': 'value'},
            data=None,
            headers={'header': 'value'}
        )

        self.assertTrue(isinstance(req_return, dict))
        self.assertFalse(req_return.get('data'))

    @mock.patch.object(requests, 'get')
    def test_get_url_get(self, _get):
        _get.return_value = 'SUCCESS'
        req = Requester('foo', 'bar')
        response = req.get_url(
            'http://dummy',
            params={'param': 'value'},
            headers=None)
        self.assertTrue(response == 'SUCCESS')

    @mock.patch.object(requests, 'post')
    def test_get_url_post(self, _post):
        _post.return_value = 'SUCCESS'
        req = Requester('foo', 'bar')
        response = req.post_url(
            'http://dummy',
            params={'param': 'value'},
            headers=None)
        self.assertTrue(response == 'SUCCESS')

    @mock.patch.object(requests, 'post')
    def test_post_xml_and_confirm_status_empty_xml(self, _post):
        _post.return_value = 'SUCCESS'
        req = Requester('foo', 'bar')
        with self.assertRaises(AssertionError) as ae:
            req.post_xml_and_confirm_status(
                url='http://dummy',
                params={'param': 'value'},
                data=None
            )

        self.assertTrue(ae.exception.message == "Unexpected type of parameter 'data': <type 'NoneType'>. Expected (str, dict)")

    @mock.patch.object(requests, 'post')
    def test_post_xml_and_confirm_status_some_xml(self, _post):
        response = requests.Response()
        response.status_code = 200
        _post.return_value = response
        req = Requester('foo', 'bar')
        ret = req.post_xml_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'},
            data='<xml/>'
        )
        self.assertTrue(isinstance(ret, requests.Response))

    @mock.patch.object(requests, 'post')
    def test_post_and_confirm_status_empty_data(self, _post):
        _post.return_value = 'SUCCESS'
        req = Requester('foo', 'bar')
        with self.assertRaises(AssertionError) as ae:
            req.post_and_confirm_status(
                url='http://dummy',
                params={'param': 'value'},
                data=None
            )

        self.assertTrue(ae.exception.message == "Unexpected type of parameter 'data': <type 'NoneType'>. Expected (str, dict)")

    @mock.patch.object(requests, 'post')
    def test_post_and_confirm_status_some_data(self, _post):
        response = requests.Response()
        response.status_code = 200
        _post.return_value = response
        req = Requester('foo', 'bar')
        ret = req.post_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'},
            data='some data'
        )
        self.assertTrue(isinstance(ret, requests.Response))

    @mock.patch.object(requests, 'post')
    def test_post_and_confirm_status_bad_result(self, _post):
        response = requests.Response()
        response.status_code = 500
        _post.return_value = response

        req = Requester('foo', 'bar')
        with self.assertRaises(JenkinsAPIException) as ae:
            req.post_and_confirm_status(
                url='http://dummy',
                params={'param': 'value'},
                data='some data'
            )

        print ae.exception.message
        self.assertTrue(ae.exception.message == "Operation failed. url=None, data=some data, headers={'Content-Type': 'application/x-www-form-urlencoded'}, status=500, text=")

    @mock.patch.object(requests, 'get')
    def test_get_and_confirm_status(self, _get):
        response = requests.Response()
        response.status_code = 200
        _get.return_value = response
        req = Requester('foo', 'bar')
        ret = req.get_and_confirm_status(
            url='http://dummy',
            params={'param': 'value'}
        )
        self.assertTrue(isinstance(ret, requests.Response))

    @mock.patch.object(requests, 'get')
    def test_get_and_confirm_status_bad_result(self, _get):
        response = requests.Response()
        response.status_code = 500
        _get.return_value = response

        req = Requester('foo', 'bar', baseurl='http://dummy')
        with self.assertRaises(JenkinsAPIException) as ae:
            req.get_and_confirm_status(
                url='http://dummy',
                params={'param': 'value'}
            )

        print ae.exception.message
        self.assertTrue(ae.exception.message == "Operation failed. url=None, headers=None, status=500, text=")

if __name__ == "__main__":
    unittest.main()
